% Simulate Mzf/Mzb1/Mzb2 in ihMT-GRE
% Authors : 
% - Lucas SOUSTELLE (lucas.soustelle@univ-amu.fr)
% - Olivier M. GIRARD (olivier.girard@univ-amu.fr)
% - Guillaume DUHAMEL (guillaume.duhamel@univ-amu.fr)
% - Andreea HERTANU (andreea.hertanu@univ-amu.fr)
% 2021/06/02
% Aix Marseille Univ, CNRS, CRMBM, Marseille, France
% Copyrights: see LICENCE file

clear; close all;
set(0,'defaulttextinterpreter','latex')
set(0,'DefaultLegendInterpreter', 'latex')
set(0,'defaultaxesfontname','cmu sans serif')
Fontsize = 18; labelsize = 24; clabel = 40;

addpath(genpath('Tools'));
gamma   = 267.513 * 1e6; % rad/s/T
COLOR_LIST_ihMTR  = {[22 27 179]/255,[0 0 0]/255,[191 11 11]/255};

%% parameters
% Tissue, delays & sequence parameters
parx.R1f        = 1/1000e-3;    % s
parx.T2f        = 0.022/parx.R1f; % s
parx.R1b        = 1/1000e-3;    % s
parx.T2b        = 10.0e-6;      % s
parx.R          = 19;           % s-1; fast "kb"
parx.M0b        = 0.16; 
parx.M0f        = 1.0; 
parx.T1D_1      = 10e-3;       % s
parx.T1D_2      = 0.4e-3;         % s
fD              = 0.5;
parx.M0b_1      = fD*parx.M0b;
parx.M0b_2      = (1-fD)*parx.M0b;
parx.D          = 1/( sqrt(15) * parx.T2b ); % rad.s-1

PREP.delta_f    = 7e3;      % Hz 
PREP.r_Tukey    = 0.3;
PREP.N_BTR      = 1;
PREP.DualType   = 'ALT'; % 'SM' / 'ALT'
PREP.N_altern   = 1;
PREP.PI         = 1-5*PREP.r_Tukey/8;
PREP.AI         = 1-PREP.r_Tukey/2;
PREP.SimuAll    = false; % simulate P/M/PM/MP ; else only P/PM
RAGE.DummyDur   = 10.0; % sec
RAGE.subTR      = 6.5e-3;        % s
RAGE.FA_nom     = 5 /180*pi;    % rad

% experiments: maintain a constant B1RMS over TR (B1RMS_TR)
dB1             = [1.0];
NSHOT           = [4 28 52];
N_C1 = 30; N_B1 = numel(dB1); N_NSHOT = numel(NSHOT);
RAGE.B1RMS_TR   = [linspace(0.25,5.0,N_C1)]*1e-6; % T
PREP.tau_sat    = [(1.0e-3*ones(1,N_C1*N_B1*N_NSHOT))];   % s
PREP.delta_t    = [(1.5e-3*ones(1,N_C1*N_B1*N_NSHOT))];   % s 
PREP.Npulses    = [(8*ones(1,N_C1*N_B1*N_NSHOT))];
PREP.BTR        = PREP.Npulses .* PREP.delta_t;

RAGE.N_subTR    = cell2mat(arrayfun(@(c) repmat(c,[1 N_C1*N_B1]),NSHOT,'UniformOutput',false));
RAGE.TR         = PREP.Npulses(1)*PREP.delta_t(1) + RAGE.N_subTR.*RAGE.subTR;
PREP.FA_sat     = repmat(RAGE.B1RMS_TR,[1 N_B1*N_NSHOT]) .*180*gamma*PREP.AI*PREP.tau_sat(1) ./ ...
                                (pi*sqrt(PREP.PI*PREP.Npulses(1)*PREP.tau_sat(1)./RAGE.TR)) .* dB1;
RAGE.FA         = repmat(RAGE.FA_nom,[1 N_C1*N_B1*N_NSHOT]);


%% Run
% B1RMS variation
tic
start_parpool('local');
parfor ii = 1 : numel(RAGE.TR)
    [PREP_par(ii), RAGE_par(ii), parx_par(ii)] = parse_par_ihMT_RAGE(PREP,RAGE,parx,ii);
    [MT0(ii),MTs_P(ii),MTd_PM(ii)] = func_simu_ihMT_RAGE(PREP_par(ii),RAGE_par(ii),parx_par(ii));
end
toc

%% plot(B1RMSTR,Mzf/Mzb1/Mzb2)
MT0_Mzf_arr     = arrayfun(@(c) MT0(c).P0_persubTR(1),1:numel(PREP.FA_sat))/cos(RAGE.FA(1));
MT0_Mzb_1_arr   = arrayfun(@(c) MT0(c).P0_persubTR(2),1:numel(PREP.FA_sat))/parx.M0b_1;
MT0_Mzb_2_arr   = arrayfun(@(c) MT0(c).P0_persubTR(4),1:numel(PREP.FA_sat))/parx.M0b_2;
MT0_beta_1_arr   = arrayfun(@(c) MT0(c).P0_persubTR(3),1:numel(PREP.FA_sat))*2*pi*PREP.delta_f;
MT0_beta_2_arr   = arrayfun(@(c) MT0(c).P0_persubTR(5),1:numel(PREP.FA_sat))*2*pi*PREP.delta_f;

MTs_Mzf_arr     = arrayfun(@(c) MTs_P(c).P0_persubTR(1),1:numel(PREP.FA_sat))/cos(RAGE.FA(1));
MTs_Mzb_1_arr   = arrayfun(@(c) MTs_P(c).P0_persubTR(2),1:numel(PREP.FA_sat))/parx.M0b_1;
MTs_Mzb_2_arr   = arrayfun(@(c) MTs_P(c).P0_persubTR(4),1:numel(PREP.FA_sat))/parx.M0b_2;
MTs_beta_1_arr   = arrayfun(@(c) MTs_P(c).P0_persubTR(3),1:numel(PREP.FA_sat))*2*pi*PREP.delta_f;
MTs_beta_2_arr   = arrayfun(@(c) MTs_P(c).P0_persubTR(5),1:numel(PREP.FA_sat))*2*pi*PREP.delta_f;

MTd_Mzf_arr     = arrayfun(@(c) MTd_PM(c).P0_persubTR(1),1:numel(PREP.FA_sat))/cos(RAGE.FA(1));
MTd_Mzb_1_arr   = arrayfun(@(c) MTd_PM(c).P0_persubTR(2),1:numel(PREP.FA_sat))/parx.M0b_1;
MTd_Mzb_2_arr   = arrayfun(@(c) MTd_PM(c).P0_persubTR(4),1:numel(PREP.FA_sat))/parx.M0b_2;
MTd_beta_1_arr   = arrayfun(@(c) MTd_PM(c).P0_persubTR(3),1:numel(PREP.FA_sat))*2*pi*PREP.delta_f;
MTd_beta_2_arr   = arrayfun(@(c) MTd_PM(c).P0_persubTR(5),1:numel(PREP.FA_sat))*2*pi*PREP.delta_f;

%%%%%%%%%%%%%% MTs
close all;
MTs_Mzf     = reshape(MTs_Mzf_arr,[N_C1 N_NSHOT]);
MTs_Mzb_1   = reshape(MTs_Mzb_1_arr,[N_C1 N_NSHOT]);
MTs_Mzb_2   = reshape(MTs_Mzb_2_arr,[N_C1 N_NSHOT]);
MTs_beta_1   = reshape(MTs_beta_1_arr,[N_C1 N_NSHOT]);
MTs_beta_2   = reshape(MTs_beta_2_arr,[N_C1 N_NSHOT]);

figure(2), set(gcf,'position',[50 50 1200 1000]); 
subplot(2,3,1)
hold on
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzf(:,1),'--','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzf(:,2),'-.','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzf(:,3),'color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot([3 3],[0 1],'--','color',[0.5 0.5 0.5],'linewidth',2)
xlabel('\fontsize{11}{0}\boldmath \textbf{$B_{1,RMS}^{TR}$ ($\mu$T)}'), xlim([0 max(RAGE.B1RMS_TR)*1e6])
ylabel('\fontsize{11}{0}\boldmath \textbf{M$_{z,f}$/M$_{0,f}$}'), ylim([0 1])
grid on
legend({'TR = 38 ms','TR = 194 ms','TR = 350 ms'},'location','southwest','fontsize',12)
set(gca,'FontSize',12)

subplot(2,3,2)
hold on
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzb_1(:,1),'--','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzb_1(:,2),'-.','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzb_1(:,3),'-','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot([3 3],[0 1],'--','color',[0.5 0.5 0.5],'linewidth',2)
ylabel('\fontsize{11}{0}\boldmath \textbf{M$_{z,b,1}$/M$_{0,b,1}$ (long $T_{1D}$)}'), ylim([0 1])
xlabel('\fontsize{11}{0}\boldmath \textbf{$B_{1,RMS}^{TR}$ ($\mu$T)}'), xlim([0 max(RAGE.B1RMS_TR)*1e6])
grid on
title('\boldmath \textbf{MT single experiment}')
set(gca,'FontSize',12)

subplot(2,3,3)
hold on
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzb_2(:,1),'--','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzb_2(:,2),'-.','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTs_Mzb_2(:,3),'-','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot([3 3],[0 1],'--','color',[0.5 0.5 0.5],'linewidth',2)
ylabel('\fontsize{11}{0}\boldmath \textbf{M$_{z,b,2}$/M$_{0,b,2}$ (short $T_{1D}$)}'), ylim([0 1])
xlabel('\fontsize{11}{0}\boldmath \textbf{$B_{1,RMS}^{TR}$ ($\mu$T)}'), xlim([0 max(RAGE.B1RMS_TR)*1e6])
grid on
set(gca,'FontSize',12)

%%%%%%%%%%%%%% MTd
MTd_Mzf     = reshape(MTd_Mzf_arr,[N_C1 N_NSHOT]);
MTd_Mzb_1   = reshape(MTd_Mzb_1_arr,[N_C1 N_NSHOT]);
MTd_Mzb_2   = reshape(MTd_Mzb_2_arr,[N_C1 N_NSHOT]);

subplot(2,3,4)
hold on
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzf(:,1),'--','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzf(:,2),'-.','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzf(:,3),'color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot([3 3],[0 1],'--','color',[0.5 0.5 0.5],'linewidth',2)
xlabel('\fontsize{11}{0}\boldmath \textbf{$B_{1,RMS}^{TR}$ ($\mu$T)}'), xlim([0 max(RAGE.B1RMS_TR)*1e6])
ylabel('\fontsize{11}{0}\boldmath \textbf{M$_{z,f}$/M$_{0,f}$}'), ylim([0 1])
grid on
set(gca,'FontSize',12)

subplot(2,3,5)
hold on
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzb_1(:,1),'--','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzb_1(:,2),'-.','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzb_1(:,3),'color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot([3 3],[0 1],'--','color',[0.5 0.5 0.5],'linewidth',2)
xlabel('\fontsize{11}{0}\boldmath \textbf{$B_{1,RMS}^{TR}$ ($\mu$T)}'), xlim([0 max(RAGE.B1RMS_TR)*1e6])
ylabel('\fontsize{11}{0}\boldmath \textbf{M$_{z,b,1}$/M$_{0,b,1}$ (long $T_{1D}$)}'), ylim([0 1])
grid on
title('\boldmath \textbf{MT dual experiment}')
set(gca,'FontSize',12)

subplot(2,3,6)
hold on
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzb_2(:,1),'--','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzb_2(:,2),'-.','color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot(RAGE.B1RMS_TR(1:N_C1)*1e6,MTd_Mzb_2(:,3),'color',COLOR_LIST_ihMTR{2},'linewidth',2)
plot([3 3],[0 1],'--','color',[0.5 0.5 0.5],'linewidth',2)
xlabel('\fontsize{11}{0}\boldmath \textbf{$B_{1,RMS}^{TR}$ ($\mu$T)}'), xlim([0 max(RAGE.B1RMS_TR)*1e6])
ylabel('\fontsize{11}{0}\boldmath \textbf{M$_{z,b,2}$/M$_{0,b,2}$ (short $T_{1D}$)}'), ylim([0 1])
grid on
set(gca,'FontSize',12)




