% Evaluate FA apparent from bloch integration
% Authors : 
% - Lucas SOUSTELLE (lucas.soustelle@univ-amu.fr)
% - Olivier M. GIRARD (olivier.girard@univ-amu.fr)
% - Guillaume DUHAMEL (guillaume.duhamel@univ-amu.fr)
% - Andreea HERTANU (andreea.hertanu@univ-amu.fr)
% 2021/06/02
% Aix Marseille Univ, CNRS, CRMBM, Marseille, France
% Copyrights: see LICENCE file

clear; close all;
set(0,'defaulttextinterpreter','latex')
set(0,'DefaultLegendInterpreter', 'latex')
set(0,'defaultaxesfontname','cmu sans serif')
Fontsize = 18; labelsize = 24; clabel = 40;

addpath(genpath('Tools'));
gamma   = 267.513 * 1e6;                    % rad/s/T
%% parameters
T2      = 22 * 1e-3; % s
delta   = linspace(4e3,15e3,501); % Hz

%% eq. FA Bloch integration single pulse
%%%%%%%%%% Tukey 0.3 PW=0.5, B1RMS=3 uT
prep.PW         = 0.5e-3;
prep.FA_sat     = 143.402; % deg
prep.r_Tukey    = 0.3;

pulse_fun   = @(t) func_TukeyFun_pulse(t,prep.PW,prep.r_Tukey);
pulse       = func_compute_pulse(pulse_fun,prep.PW,prep.FA_sat,0);
FA_DS_Tukeyr03_PW05 = zeros(size(delta));
parfor ii = 1 : numel(delta)
    pulse_tmp       = pulse;
    pulse_tmp.delta = delta(ii);
    Mz              = func_compute_DS_noMT(T2, pulse_tmp);
    FA_DS_Tukeyr03_PW05(ii) = acosd(Mz);
end

%%%%%%%%%% Tukey 0.3 PW=1.0, B1RMS=3 uT
prep.PW         = 1.0e-3;
prep.FA_sat     = 287; % deg
prep.r_Tukey    = 0.3;

pulse_fun   = @(t) func_TukeyFun_pulse(t,prep.PW,prep.r_Tukey);
pulse       = func_compute_pulse(pulse_fun,prep.PW,prep.FA_sat,0);
FA_DS_Tukeyr03_PW1 = zeros(size(delta));
parfor ii = 1 : numel(delta)
    pulse_tmp       = pulse;
    pulse_tmp.delta = delta(ii);
    Mz              = func_compute_DS_noMT(T2, pulse_tmp);
    FA_DS_Tukeyr03_PW1(ii) = acosd(Mz);
end

%% plot
close all
COLOR_LIST_ihMTR  = {[191 11 11]/255,[0 0 0]/255};
figure(1), set(gcf,'position',[50 50 1400 700]); 
hold on
plot(delta/1e3,FA_DS_Tukeyr03_PW1,'linewidth',2,'color',COLOR_LIST_ihMTR{1})
plot(delta/1e3,FA_DS_Tukeyr03_PW05,'linewidth',2,'color',COLOR_LIST_ihMTR{2})
plot([5 5 ; 7 7 ; 8 8 ; 10.4 10.4 ; 12 12]',[0 20 ; 0 20 ; 0 20 ; 0 20 ; 0 20]','--','color',[0.5 0.5 0.5],'linewidth',2)

ylabel('\fontsize{11}{0}\boldmath \textbf{Equivalent on-resonance FA (deg)}')
xlabel('\fontsize{11}{0}\boldmath \textbf{$\Delta$f (kHz)}')
legend({'$r=0.3$ --- $\textrm{PW}=1.0$ ms','$r=0.3$ --- $\textrm{PW}=0.5$ ms'},'location','northeast','fontsize',16)
xticks(linspace(4,15,12))
axis([delta(1)/1e3 delta(end)/1e3 0 20])
grid on
set(gca,'FontSize',14)


