% Bloch equation integration without MT/T1
% Authors : 
% - Lucas SOUSTELLE (lucas.soustelle@univ-amu.fr)
% - Olivier M. GIRARD (olivier.girard@univ-amu.fr)
% - Guillaume DUHAMEL (guillaume.duhamel@univ-amu.fr)
% - Andreea HERTANU (andreea.hertanu@univ-amu.fr)
% Aix Marseille Univ, CNRS, CRMBM, Marseille, France
% Copyrights: see LICENCE file

function [Mz,M_temp] = func_compute_DS_noMT(T2, pulse, M0)
% Simulated saturation effect of MT pulse on free pool
if ~exist('M0','var'); M0 = [0 0 1]; end
options = odeset('RelTol',1e-10,'AbsTol',1e-7);
[~, M_temp]     = ode23(@(t,M) BlochEq_NoMT(t, M, T2, pulse), [0 pulse.tau], M0,options);
Mz              = M_temp(end,3);

end


function dM = BlochEq_NoMT(t, M, T2, Pulse)
    dM    =  zeros(3,1);
    omega =  Pulse.Omega1(t);
    delta =  Pulse.delta;

    dM(1) = - M(1)/T2 - 2*pi*delta*M(2);
    dM(2) = - M(2)/T2 + 2*pi*delta*M(1) + omega*M(3);
    dM(3) = - omega*M(2);
end

