% Compute pulse characteristics
% Authors : 
% - Lucas SOUSTELLE (lucas.soustelle@univ-amu.fr)
% - Olivier M. GIRARD (olivier.girard@univ-amu.fr)
% - Guillaume DUHAMEL (guillaume.duhamel@univ-amu.fr)
% - Andreea HERTANU (andreea.hertanu@univ-amu.fr)
% Aix Marseille Univ, CNRS, CRMBM, Marseille, France
% Copyrights: see LICENCE file

function [MT0,MTs_P,MTd_PM,MTs_M,MTd_MP] = func_simu_ihMT_RAGE(PREP,RAGE,parx)

% DEFINE DELAYS
EVE.SAT_RD.dur      = PREP.tau_sat; % no sat. pulse for MT0
EVE.DT_RD.dur       = PREP.delta_t-PREP.tau_sat;
EVE.BTR_RD.dur      = PREP.BTR-PREP.Npulses*PREP.delta_t;
EVE.RAGE_RD.dur     = RAGE.subTR;
EVE.RAGE_POST.dur   = RAGE.TR - (PREP.N_BTR*PREP.BTR+RAGE.N_subTR*RAGE.subTR);

% BASIC MT MATRICES
R_Zn = [    -(parx.R1f+parx.R*parx.M0b),    parx.R*parx.M0f,                0,              parx.R*parx.M0f,             0; ...
            parx.R*parx.M0b_1,              -(parx.R1b+parx.R*parx.M0f),    0,              0,                           0; ...
            0,                              0,                              -1/parx.T1D_1,  0,                           0; ...
            parx.R*parx.M0b_2,              0,                              0,              -(parx.R1b+parx.R*parx.M0f), 0; ...
            0,                              0,                              0,              0,                           -1/parx.T1D_2];
[W_SM,W_P,W_M]  = build_W(PREP,parx); % Wb saturation matrices
C               = [parx.R1f*parx.M0f parx.R1b*parx.M0b_1 0 parx.R1b*parx.M0b_2 0]';    

% SATURATION PULSE: MT with direct saturation (Wf)
DS_P   = blkdiag(-PREP.Wf_P,zeros(4,4)); % P/M case
DS_M   = blkdiag(-PREP.Wf_M,zeros(4,4)); 
DS_SM  = zeros(5,5); % S/M case: consider no residual on-res rotation

% SATURATION PULSE: OPERATORS
SAT.TZFn_SM   =  expm( (DS_SM+R_Zn+W_SM)*PREP.tau_sat );
SAT.TZF0_SM   = (expm( (DS_SM+R_Zn+W_SM)*PREP.tau_sat )    - eye(5)) * ((DS_SM+R_Zn+W_SM)\C);

SAT.TZFn_P    =  expm( (DS_P+R_Zn+W_P)*PREP.tau_sat );
SAT.TZF0_P    = (expm( (DS_P+R_Zn+W_P)*PREP.tau_sat )      - eye(5)) * ((DS_P+R_Zn+W_P)\C);

SAT.TZFn_M    =  expm( (DS_M+R_Zn+W_M)*PREP.tau_sat );
SAT.TZF0_M    = (expm( (DS_M+R_Zn+W_M)*PREP.tau_sat )      - eye(5)) * ((DS_M+R_Zn+W_M)\C);

% RELAXATION SAT_RD, DT_RD, BTR_RD, RAGE_RD & RAGE_POST
EVE.SAT_RD      = RELAX_matrices(EVE.SAT_RD,R_Zn,C);
EVE.DT_RD       = RELAX_matrices(EVE.DT_RD,R_Zn,C);
EVE.BTR_RD      = RELAX_matrices(EVE.BTR_RD,R_Zn,C);
EVE.RAGE_RD     = RELAX_matrices(EVE.RAGE_RD,R_Zn,C);
EVE.RAGE_POST   = RELAX_matrices(EVE.RAGE_POST,R_Zn,C);


%% SEQ LOOPS
switch PREP.DualType
case 'SM'
    MTd_PM = SEQ_LOOP(PREP,RAGE,parx,EVE,SAT.TZFn_SM,SAT.TZF0_SM);
case 'ALT'
    MTd_PM = SEQ_LOOP(PREP,RAGE,parx,EVE,SAT.TZFn_P,SAT.TZF0_P,SAT.TZFn_M,SAT.TZF0_M);
    if PREP.SimuAll
        MTd_MP = SEQ_LOOP(PREP,RAGE,parx,EVE,SAT.TZFn_M,SAT.TZF0_M,SAT.TZFn_P,SAT.TZF0_P);
    end
end

% SINGLE (same TZF* matrices for "Plus" & "Minus")
MTs_P = SEQ_LOOP(PREP,RAGE,parx,EVE,SAT.TZFn_P,SAT.TZF0_P,SAT.TZFn_P,SAT.TZF0_P);
if PREP.SimuAll
    MTs_M = SEQ_LOOP(PREP,RAGE,parx,EVE,SAT.TZFn_M,SAT.TZF0_M,SAT.TZFn_M,SAT.TZF0_M);
end

% MT0
MT0 = SEQ_LOOP(PREP,RAGE,parx,EVE);



end

function STORE = SEQ_LOOP(PREP,RAGE,parx,EVE,SAT_TZFn_1,SAT_TZF0_1,SAT_TZFn_2,SAT_TZF0_2)
    STORE               = struct();
    Omega               = [parx.M0f ; parx.M0b_1 ; 0 ; parx.M0b_2 ; 0];  % initial state
    
    STORE.P0_persubTR   = complex(zeros(5,RAGE.N_subTR),zeros(5,RAGE.N_subTR));
    STORE.Z0_persubTR   = complex(zeros(5,RAGE.N_subTR),zeros(5,RAGE.N_subTR));
    STORE.Z0_perTR      = complex(zeros(1,RAGE.N_TR),zeros(1,RAGE.N_TR));
    
    for kk = 1 : RAGE.N_TR

        % ihMT PREPARATION 
        for pp = 1 : PREP.N_BTR
            switch nargin
            case 8 % ALT AND SINGLE case
                for ii = 1 : PREP.Npulses/PREP.N_altern/2
                    for tt = 1 : PREP.N_altern % Plus
                        % saturation
                        Omega = OPERATOR_SATURATION_ROTATION(Omega,SAT_TZFn_1,SAT_TZF0_1);

                        % delta_t resting delay 
                        Omega = OPERATOR_RELAXATION_MT(Omega,EVE.DT_RD.TZn,EVE.DT_RD.TZ0);
                    end
                    for tt = 1 : PREP.N_altern % Minus in ALT case; Plus in SINGLE case
                        % saturation
                        Omega = OPERATOR_SATURATION_ROTATION(Omega,SAT_TZFn_2,SAT_TZF0_2);

                        % delta_t resting delay 
                        Omega = OPERATOR_RELAXATION_MT(Omega,EVE.DT_RD.TZn,EVE.DT_RD.TZ0);
                    end
                end
            case 6 % SM case
                for ii = 1 : PREP.Npulses
                    % saturation
                    Omega = OPERATOR_SATURATION_ROTATION(Omega,SAT_TZFn_1,SAT_TZF0_1);

                    % delta_t resting delay 
                    Omega = OPERATOR_RELAXATION_MT(Omega,EVE.DT_RD.TZn,EVE.DT_RD.TZ0);
                end
            case 4 % MT0 case
                for ii = 1 : PREP.Npulses
                    % saturation
                    Omega = OPERATOR_RELAXATION_MT(Omega,EVE.SAT_RD.TZn,EVE.SAT_RD.TZ0);
                    
                    % delta_t resting delay 
                    Omega = OPERATOR_RELAXATION_MT(Omega,EVE.DT_RD.TZn,EVE.DT_RD.TZ0);
                end
            end
            
            % BTR resting delay
            Omega = OPERATOR_RELAXATION_MT(Omega,EVE.BTR_RD.TZn,EVE.BTR_RD.TZ0);
        end

        % RAGE READOUT
        for pp = 1 : RAGE.N_subTR
            % instant. rotation
            Omega = OPERATOR_ROTATION(Omega,RAGE.FA);
            
            % store quantities
            if pp == 1, STORE.Z0_perTR(kk) = Omega(1); end
            STORE.Z0_persubTR(pp)                   = Omega(1);
            STORE.P0_persubTR(:,pp)                 = Omega(:);
            
            % sub-TR resting delay
            Omega = OPERATOR_RELAXATION_MT(Omega,EVE.RAGE_RD.TZn,EVE.RAGE_RD.TZ0);
        end

        % RELAX/DIFF TO TR
        Omega = OPERATOR_RELAXATION_MT(Omega,EVE.RAGE_POST.TZn,EVE.RAGE_POST.TZ0);
    end 
end


%% BUILD MATRICES
% W SATURATION
function [W_SM,W_P,W_M] = build_W(PREP,parx)
    delta_f_P = PREP.delta_f+parx.dB0;
    delta_f_M = PREP.delta_f-parx.dB0; % "absolute" values, already signed in W_M matrix

    % SM
    W_SM = [0,  0,                                      0,                                      0,                                      0; ...
            0,  -PREP.Wb,                               0,                                      0,                                      0; ...
            0,  0,                                      -PREP.Wb*(2*pi*PREP.delta_f/parx.D)^2,0,                                        0; ... 
            0,  0,                                      0,                                      -PREP.Wb,                               0; ...
            0,  0,                                      0,                                      0,                                      -PREP.Wb*(2*pi*PREP.delta_f/parx.D)^2];
        
    % Plus
    W_P = [ 0,  0,                                      0,                                      0,                                      0; ...
            0,  -PREP.Wb_P,                             PREP.Wb_P*2*pi*delta_f_P,               0,                                      0; ...
            0,  PREP.Wb_P*2*pi*delta_f_P/parx.D^2,      -PREP.Wb_P*(2*pi*delta_f_P/parx.D)^2,   0,                                      0; ... 
            0,  0,                                      0,                                      -PREP.Wb_P,                             PREP.Wb_P*2*pi*delta_f_P; ...
            0,  0,                                      0,                                      PREP.Wb_P*2*pi*delta_f_P/parx.D^2,      -PREP.Wb_P*(2*pi*delta_f_P/parx.D)^2];
    % Minus
    W_M = [ 0,  0,                                      0,                                      0,                                      0; ...
            0,  -PREP.Wb_M,                             -PREP.Wb_M*2*pi*delta_f_M,              0,                                      0; ...
            0,  -PREP.Wb_M*2*pi*delta_f_M/parx.D^2,     -PREP.Wb_M*(2*pi*delta_f_M/parx.D)^2,   0,                                      0; ... 
            0,  0,                                      0,                                      -PREP.Wb_M,                             -PREP.Wb_M*2*pi*delta_f_M; ...
            0,  0,                                      0,                                      -PREP.Wb_M*2*pi*delta_f_M/parx.D^2,     -PREP.Wb_M*(2*pi*delta_f_M/parx.D)^2];
end

% RELAXATION 
function STRUCT_RELAX = RELAX_matrices(STRUCT_RELAX,R_Zn,C)
    STRUCT_RELAX.TZn  = expm(  R_Zn*STRUCT_RELAX.dur );
    STRUCT_RELAX.TZ0  = (expm( R_Zn*STRUCT_RELAX.dur )  - eye(numel(C))) * (R_Zn\C);
end

%% OPERATORS
function Omega = OPERATOR_RELAXATION_MT(Omega,R_Zn,R_Z0)
    Omega  = R_Zn * Omega + R_Z0;
end

function Omega = OPERATOR_ROTATION(Omega,fa) % tilt Mzf only
    Omega(1)  = Omega(1)*cos(fa);
end

function Omega = OPERATOR_SATURATION_ROTATION(Omega,SR_Zn,SR_Z0)
    Omega       = SR_Zn * Omega + SR_Z0;
end


