% Parse ihMT-RAGE sequence parameters for simulations
% Authors : 
% - Lucas SOUSTELLE (lucas.soustelle@univ-amu.fr)
% - Olivier M. GIRARD (olivier.girard@univ-amu.fr)
% - Guillaume DUHAMEL (guillaume.duhamel@univ-amu.fr)
% - Andreea HERTANU (andreea.hertanu@univ-amu.fr)
% Aix Marseille Univ, CNRS, CRMBM, Marseille, France
% Copyrights: see LICENCE file

function [PREP_par, RAGE_par, parx_par] = parse_par_ihMT_RAGE(PREP,RAGE,parx,ii)
gamma   = 267.513 * 1e6;

% check typical missing fields
if ~isfield(parx,'dB0'), parx.dB0 = 0; end

% initialize
PREP_par = PREP;
RAGE_par = RAGE;
parx_par = parx;

if numel(PREP.delta_f) >= ii,   PREP_par.delta_f    = PREP.delta_f(ii); end
if numel(PREP.tau_sat) >= ii,   PREP_par.tau_sat    = PREP.tau_sat(ii); end
if numel(PREP.delta_t) >= ii,   PREP_par.delta_t    = PREP.delta_t(ii); end
if numel(PREP.FA_sat) >= ii,    PREP_par.FA_sat     = PREP.FA_sat(ii); end
if numel(PREP.Npulses) >= ii,   PREP_par.Npulses    = PREP.Npulses(ii); end
if numel(PREP.BTR) >= ii,       PREP_par.BTR        = PREP.BTR(ii); end
if numel(RAGE.FA) >= ii,        RAGE_par.FA         = RAGE.FA(ii); end
if numel(RAGE.TR) >= ii,        RAGE_par.TR         = RAGE.TR(ii); end
if numel(RAGE.N_subTR) >= ii,   RAGE_par.N_subTR    = RAGE.N_subTR(ii); end


% PREP
pulse_fun           = @(t) func_TukeyFun_pulse(t,PREP_par.tau_sat,PREP_par.r_Tukey);
PREP_par.pulse      = func_compute_pulse(pulse_fun,PREP_par.tau_sat,PREP_par.FA_sat,PREP_par.delta_f);
PREP_par.Wb         = func_computeW(PREP_par.pulse.w1RMS,PREP_par.delta_f,parx.T2b,'SuperLorentzian');
PREP_par.Wf         = func_computeW(PREP_par.pulse.w1RMS,PREP_par.delta_f,parx.T2f,'Lorentzian');
if parx_par.dB0 == 0
    PREP_par.Wb_P       = PREP_par.Wb;
    PREP_par.Wb_M       = PREP_par.Wb;
    PREP_par.Wf_P       = PREP_par.Wf; 
    PREP_par.Wf_M       = PREP_par.Wf; 
else   
    PREP_par.Wb_P       = func_computeW(PREP_par.pulse.w1RMS, PREP_par.delta_f+parx_par.dB0,parx.T2b,'SuperLorentzian');
    PREP_par.Wb_M       = func_computeW(PREP_par.pulse.w1RMS,-PREP_par.delta_f+parx_par.dB0,parx.T2b,'SuperLorentzian');
    PREP_par.Wf_P       = func_computeW(PREP_par.pulse.w1RMS, PREP_par.delta_f+parx_par.dB0,parx.T2f,'Lorentzian');
    PREP_par.Wf_M       = func_computeW(PREP_par.pulse.w1RMS,-PREP_par.delta_f+parx_par.dB0,parx.T2f,'Lorentzian');
end

% RAGE
RAGE_par.N_TR           = ceil(RAGE.DummyDur * parx.R1f/RAGE_par.TR);

% info
PREP_par.info_B1RMSTR_uT = PREP_par.pulse.w1RMS/gamma*1e6*sqrt(PREP_par.Npulses*PREP_par.tau_sat/RAGE_par.TR);
PREP_par.info_B1peak_uT  = PREP_par.pulse.B1peak*1e6;

PREP_par.FA_app          = acos(func_compute_DS_noMT(parx.T2f, PREP_par.pulse)); % rad
PREP_par.info_FA_app_deg = PREP_par.FA_app*180/pi;